/*

 File: taylor.h, 2004/06/14

 ACETAF, Version 3.0.0

 Copyright (C) Markus Neher, markus.neher@math.uni-karlsruhe.de
               Ingo Eble,    ingoeble@web.de

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _TAYLOR_H_INCLUDED
#define _TAYLOR_H_INCLUDED

//C++ standard headers
#include <iostream>
#include <string>
#include <vector>

#include "factorialtable.h"

template<class T> class Taylor
{
 public:

  /*
    Ctors
  */

  Taylor() : cont_(0) {}
  Taylor(int,unsigned int =0,unsigned int =0);
  Taylor(const double&,unsigned int =0,unsigned int =0);
  Taylor(const T&,unsigned int =0,unsigned int =0);
  ~Taylor() {}

  /*
    Access to components
  */

  int order() const { return cont_.size()-1; }

  T&       operator [](unsigned int i)       { return cont_[i]; }
  const T& operator [](unsigned int i) const { return cont_[i]; }
  
  const T derivative(unsigned int i) const
    { return cont_[i] * Table_of_Factorials::GetTable().factorial(i); }
 
  /*
    Arithmetik operations.
  */

  Taylor operator - () const; 
  
  Taylor& operator += (const Taylor&);
  Taylor& operator -= (const Taylor&);
  Taylor& operator *= (const Taylor&);
  Taylor& operator /= (const Taylor&);

  Taylor& operator += (const T&);
  Taylor& operator -= (const T&);
  Taylor& operator *= (const T&);
  Taylor& operator /= (const T&);

  /*
    Function for resizing.
  */

  void change_order(unsigned int n) { cont_.resize(n+1,(T)0.0); }

  /*
    Friend functions
  */

  friend Taylor sqrt <>(const Taylor&);
  friend Taylor sqrt <>(const Taylor&,int);
  friend Taylor sqr  <>(const Taylor&); 
  friend Taylor pow  <>(const Taylor&,const T&);
  friend Taylor pow  <>(const Taylor&,const Taylor&);
  friend Taylor power<>(const Taylor&,int);
  friend Taylor exp  <>(const Taylor&);
  friend Taylor ln   <>(const Taylor&);
  friend Taylor sin  <>(const Taylor&);
  friend Taylor cos  <>(const Taylor&);
  friend Taylor tan  <>(const Taylor&);
  friend Taylor cot  <>(const Taylor&);
  friend Taylor sinh <>(const Taylor&);
  friend Taylor cosh <>(const Taylor&);
  friend Taylor tanh <>(const Taylor&);
  friend Taylor coth <>(const Taylor&);
  friend Taylor asin <>(const Taylor&);
  friend Taylor acos <>(const Taylor&);
  friend Taylor atan <>(const Taylor&);
  friend Taylor acot <>(const Taylor&);
  friend Taylor asinh<>(const Taylor&);
  friend Taylor acosh<>(const Taylor&);
  friend Taylor atanh<>(const Taylor&);
  friend Taylor acoth<>(const Taylor&);

  friend void f_g_u<>(Taylor&,const Taylor&,const Taylor&,int);  //Auxiliary function

  friend std::ostream& operator << <>(std::ostream&,const Taylor&);
  friend std::istream& operator >> <>(std::istream&,      Taylor&);
  friend std::string&  operator << <>(std::string&, const Taylor&);
  friend std::string&  operator >> <>(std::string&,       Taylor&);


 private:

  std::vector<T> cont_; //Container for Taylor coefficients.
};

#include "taylor.tpl"

template<class T> inline int Order(const Taylor<T>& x) { return x.order(); }

template<class T> const Taylor<T> operator + (const Taylor<T>& x,const Taylor<T>& y) 
{ 
  Taylor<T> result(x);
  return result += y; 
}
template<class T> const Taylor<T> operator - (const Taylor<T>& x,const Taylor<T>& y)
{ 
  Taylor<T> result(x);
  return result -= y; 
}
template<class T> const Taylor<T> operator * (const Taylor<T>& x,const Taylor<T>& y)
{ 
  Taylor<T> result(x);
  return result *= y; 
}
template<class T> const Taylor<T> operator / (const Taylor<T>& x,const Taylor<T>& y)
{ 
  Taylor<T> result(x);
  return result /= y; 
}

template<class T> const Taylor<T> operator + (const T& d,const Taylor<T>& x)
{
  Taylor<T> result(d,0,x.order());
  return result += x; 
}
template<class T> const Taylor<T> operator - (const T& d,const Taylor<T>& x)
{
  Taylor<T> result(d,0,x.order());
  return result -= x; 
}
template<class T> const Taylor<T> operator * (const T& d,const Taylor<T>& x)
{
  Taylor<T> result(d,0,x.order());
  return result *= x; 
}
template<class T> const Taylor<T> operator / (const T& d,const Taylor<T>& x)
{
  Taylor<T> result(d,0,x.order());
  return result /= x; 
}

template<class T> const Taylor<T> operator + (const Taylor<T>& x,const T& d)
{
  Taylor<T> result(x);
  return result += d; 
}
template<class T> const Taylor<T> operator - (const Taylor<T>& x,const T& d)
{
  Taylor<T> result(x);
  return result -= d; 
}
template<class T> const Taylor<T> operator * (const Taylor<T>& x,const T& d)
{
  Taylor<T> result(x);
  return result *= d; 
}
template<class T> const Taylor<T> operator / (const Taylor<T>& x,const T& d)
{
  Taylor<T> result(x);
  return result /= d; 
}


template<class T> class TaylorMemory
{
  Taylor<T> memory;

 public:

  TaylorMemory() {}
  explicit TaylorMemory(const Taylor<T>& tayl) : memory( tayl ) {}
  TaylorMemory(const TaylorMemory& tm) : memory( tm.memory ) {}
  ~TaylorMemory() {}

  TaylorMemory<T>& operator = (const TaylorMemory<T>& tm)
  {
    if( this == &tm ) return *this;

    memory = tm.memory;

    return *this;
  }

  TaylorMemory<T>& operator = (const Taylor<T>& tayl)
  {
    if( memory.order() < tayl.order() )
      memory = tayl;
    return *this;
  }

  const Taylor<T>&  getTaylor() const { return memory; }
};



// Some typedefs.
//

#include "adaptintval.h"
#include "cinterval.h"

using Adapt::Interval;

typedef Taylor<Interval>   ITaylor;
typedef Taylor<CInterval>  CITaylor;

//
//--------------------------------------------------------------------

#endif

/*

  End of File: taylor.h

*/
